@@ -20,16 +20,6 @@ DIE=0
   DIE=1
 }
 
-(grep "^IT_PROG_INTLTOOL" $srcdir/configure.ac >/dev/null) && {
-  (intltoolize --version) < /dev/null > /dev/null 2>&1 || {
-    echo 
-    echo "**Error**: You must have \`intltool' installed."
-    echo "You can get it from:"
-    echo "  ftp://ftp.gnome.org/pub/GNOME/stable/sources/intltool/"
-    DIE=1
-  }
-}
-
 (grep "^LT_INIT" $srcdir/configure.ac >/dev/null) && {
   (libtoolize --version) < /dev/null > /dev/null 2>&1 || {
     echo
@@ -57,6 +47,14 @@ grep "^AM_GLIB_GNU_GETTEXT" $srcdir/configure.ac >/dev/null && {
   NO_AUTOMAKE=yes
 }
 
+test -n "$NO_AUTOMAKE" || (autopoint --version) < /dev/null > /dev/null 2>&1 || {
+  echo
+  echo "**Error**: You must have \`autopoint' installed."
+  echo "You can get it from: ftp://ftp.gnu.org/pub/gnu/"
+  DIE=1
+  NO_AUTOPOINT=yes
+}
+
 
 # if no automake, don't bother testing for aclocal
 test -n "$NO_AUTOMAKE" || (aclocal --version) < /dev/null > /dev/null 2>&1 || {
@@ -102,16 +100,14 @@ do
 	echo "Making $dr/aclocal.m4 writable ..."
 	test -r $dr/aclocal.m4 && chmod u+w $dr/aclocal.m4
       fi
-      if grep "^IT_PROG_INTLTOOL" configure.ac >/dev/null; then
-        echo "Running intltoolize..."
-	intltoolize --copy --force --automake
-      fi
       if grep "^LT_INIT" configure.ac >/dev/null; then
 	if test -z "$NO_LIBTOOLIZE" ; then 
 	  echo "Running libtoolize..."
 	  libtoolize --force --copy
 	fi
       fi
+      echo "Running autopoint..."
+      autopoint -f
       echo "Running aclocal $aclocalinclude ..."
       aclocal $aclocalinclude
       if grep "^AC_CONFIG_HEADERS" configure.ac >/dev/null; then
