@@ -197,7 +197,7 @@ RootFunction::RootFunction() : MathFunction("root", 2) {
 	arg->setComplexAllowed(false);
 	arg->setHandleVector(true);
 	setArgumentDefinition(1, arg);
-	NumberArgument *arg2 = new NumberArgument("", ARGUMENT_MIN_MAX_NONE, true, false);
+	NumberArgument *arg2 = new NumberArgument("", ARGUMENT_MIN_MAX_NONZERO, true, false);
 	arg2->setComplexAllowed(false);
 	arg2->setRationalNumber(true);
 	arg2->setHandleVector(true);
@@ -205,7 +205,7 @@ RootFunction::RootFunction() : MathFunction("root", 2) {
 }
 int RootFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
 	if(vargs[0].isVector()) return 0;
-	if(vargs[1].number().isOne()) {
+	if(vargs[1].number().isOne() || vargs[0].isOne()) {
 		mstruct = vargs[0];
 		return 1;
 	}
@@ -690,6 +690,18 @@ int LogFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, c
 			}
 		}
 	}
+	MathStructure *mtest = &mstruct;
+	if(mtest->isPower()) mtest = mtest->exponent();
+	bool unit_error = mtest->isUnit_exp();
+	if(mtest->isMultiplication()) {
+		for(size_t i = 0; i < mtest->size(); i++) {
+			if(mtest->getChild(i + 1)->isUnit_exp()) {
+				unit_error = true;
+				break;
+			}
+		}
+	}
+	if(unit_error) CALCULATOR->error(false, _("Unexpected unit in logarithm."), NULL);
 	if(eo.allow_complex && mstruct.representsNegative()) {
 		mstruct.negate();
 		mstruct.transformById(FUNCTION_ID_LOG);
@@ -763,9 +775,21 @@ int LognFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		}
 	} else if(mstruct.isNumber() && mstructv2.isNumber()) {
 		Number nr(mstruct.number());
-		if(nr.log(mstructv2.number()) && !(eo.approximation == APPROXIMATION_EXACT && nr.isApproximate() && !mstruct.isApproximate()) && !(!eo.allow_complex && nr.isComplex() && !mstruct.number().isComplex()) && !(!eo.allow_infinite && nr.includesInfinity() && !mstruct.number().includesInfinity())) {
-			mstruct.set(nr, true);
-			return 1;
+		if(nr.log(mstructv2.number())) {
+			if(eo.approximation != APPROXIMATION_APPROXIMATE && !mstruct.isApproximate() && !mstructv2.isApproximate() && nr.isApproximate() && (eo.approximation == APPROXIMATION_EXACT || (nr < 100 && nr > -100)) && !nr.isNonInteger()) {
+				Number nr2;
+				if(nr.getCentralInteger(nr2)) {
+					Number nr_test(mstructv2.number());
+					nr_test.raise(nr2, eo.approximation == APPROXIMATION_EXACT);
+					if(mstruct.number().equals(nr_test)) {
+						nr = nr2;
+					}
+				}
+			}
+			if(!(eo.approximation == APPROXIMATION_EXACT && nr.isApproximate() && !mstruct.isApproximate()) && !(!eo.allow_complex && nr.isComplex() && !mstruct.number().isComplex()) && !(!eo.allow_infinite && nr.includesInfinity() && !mstruct.number().includesInfinity())) {
+				mstruct.set(nr, true);
+				return 1;
+			}
 		}
 	}
 	mstruct.set(CALCULATOR->getFunctionById(FUNCTION_ID_LOG), &vargs[0], NULL);
