@@ -34,7 +34,6 @@
 #define PREPEND_REF(o)		{MathStructure *m_append = o; v_order.insert(v_order.begin(), v_subs.size()); v_subs.push_back(m_append); m_append->ref(); if(!b_approx && m_append->isApproximate()) b_approx = true; if(m_append->precision() > 0 && (i_precision < 1 || m_append->precision() < i_precision)) i_precision = m_append->precision();}
 #define INSERT_REF(o, i)	{MathStructure *m_append = o; v_order.insert(v_order.begin() + i, v_subs.size()); v_subs.push_back(m_append); m_append->ref(); if(!b_approx && m_append->isApproximate()) b_approx = true; if(m_append->precision() > 0 && (i_precision < 1 || m_append->precision() < i_precision)) i_precision = m_append->precision();}
 #define CLEAR			v_order.clear(); for(size_t i = 0; i < v_subs.size(); i++) {v_subs[i]->unref();} v_subs.clear();
-//#define REDUCE(v_size)		for(size_t v_index = v_size; v_index < v_order.size(); v_index++) {v_subs[v_order[v_index]]->unref(); v_subs.erase(v_subs.begin() + v_order[v_index]);} v_order.resize(v_size);
 #define REDUCE(v_size)          {\
 	for(size_t v_index = v_size; v_index < v_order.size(); v_index++) {\
 		v_subs[v_order[v_index]]->unref();\
@@ -64,7 +63,17 @@
 #define CHILD(v_index)		(*v_subs[v_order[v_index]])
 #define SIZE			v_order.size()
 #define LAST			(*v_subs[v_order[v_order.size() - 1]])
-#define ERASE(v_index)		v_subs[v_order[v_index]]->unref(); v_subs.erase(v_subs.begin() + v_order[v_index]); for(size_t v_index2 = 0; v_index2 < v_order.size(); v_index2++) {if(v_order[v_index2] > v_order[v_index]) v_order[v_index2]--;} v_order.erase(v_order.begin() + (v_index));
+#define ERASE(v_index)		{\
+	size_t i_order = v_order[v_index];\
+	v_subs[i_order]->unref();\
+	v_subs.erase(v_subs.begin() + i_order);\
+	if(i_order < v_subs.size()) {\
+		for(std::vector<size_t>::iterator it = v_order.begin(); it != v_order.end(); ++it) {\
+			if(*it > i_order) (*it)--;\
+		}\
+	}\
+	v_order.erase(v_order.begin() + (v_index));\
+}
 
 #define IS_REAL(o)		(o.isNumber() && o.number().isReal())
 #define IS_RATIONAL(o)		(o.isNumber() && o.number().isRational())
@@ -115,6 +124,14 @@
 #define INTERNAL_ID_R_CH '\x03'
 #define INTERNAL_ID_LR "\x02\x03"
 
+#define VARIABLE_APPROXIMATE(xv) (xv->isApproximate() || ((KnownVariable*) xv)->get().isApproximate() || ((KnownVariable*) xv)->get().containsInterval(true, false, false, 0, true))
+#define SET_VARIABLE_VALUE(xm, xv, xeo) xm.set(((KnownVariable*) xv)->get()); \
+					xm.unformat(xeo); \
+					fix_intervals(xm, xeo, NULL, PRECISION);
+#define SET_VARIABLE_VALUE_THIS(xeo)	set(((KnownVariable*) o_variable)->get()); \
+					unformat(xeo); \
+					fix_intervals(*this, xeo, NULL, PRECISION);
+
 void printRecursive(const MathStructure &mstruct);
 
 std::string format_and_print(const MathStructure &mstruct);
@@ -197,7 +214,8 @@ bool comparison_is_equal_or_less(ComparisonResult cr);
 bool comparison_is_equal_or_greater(ComparisonResult cr);
 bool comparison_might_be_equal(ComparisonResult cr);
 Variable *find_interval_replace_var_comp(MathStructure &m, const EvaluationOptions &eo, Variable **v);
-
+bool represents_loose_matrix(const MathStructure &m);
+void fix_user_function_expression(std::string &str, const EvaluationOptions &eo);
 void generate_plotvector(const MathStructure &m, MathStructure x_mstruct, const MathStructure &min, const MathStructure &max, int steps, MathStructure &x_vector, MathStructure &y_vector, const EvaluationOptions &eo, bool adaptive = true);
 void generate_plotvector(const MathStructure &m, MathStructure x_mstruct, const MathStructure &min, const MathStructure &max, const MathStructure &step, MathStructure &x_vector, MathStructure &y_vector, const EvaluationOptions &eo);
 
