@@ -1034,6 +1034,7 @@ int checkAvailableVersion(const char *version_id, const char *current_version, s
 		std::vector<int> version_parts_old, version_parts_new;
 
 		string s_old_version = current_version;
+		if(s_old_version == "5.5.1" && s_version == "5.5.2") return 0;
 		while((i = s_old_version.find('.', 0)) != string::npos) {
 			version_parts_old.push_back(s2i(s_old_version.substr(0, i)));
 			s_old_version = s_old_version.substr(i + 1);
@@ -1070,7 +1071,9 @@ int checkAvailableVersion(const char *version_id, const char *current_version, s
 }
 
 void free_thread_caches() {
+#if MPFR_VERSION_MAJOR >= 4
 	mpfr_free_cache2(MPFR_FREE_LOCAL_CACHE);
+#endif
 }
 
 #ifdef _WIN32
@@ -1094,7 +1097,9 @@ DWORD WINAPI Thread::doRun(void *data) {
 	Thread *thread = (Thread *) data;
 	SetEvent(thread->m_threadReadyEvent);
 	thread->run();
+#if MPFR_VERSION_MAJOR >= 4
 	mpfr_free_cache2(MPFR_FREE_LOCAL_CACHE);
+#endif
 	thread->running = false;
 	return 0;
 }
@@ -1144,7 +1149,9 @@ Thread::~Thread() {
 void Thread::doCleanup(void *data) {
 	Thread *thread = (Thread *) data;
 	thread->running = false;
+#if MPFR_VERSION_MAJOR >= 4
 	mpfr_free_cache2(MPFR_FREE_LOCAL_CACHE);
+#endif
 }
 
 void Thread::enableAsynchronousCancel() {
